@@ -1,6 +1,6 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 #@PydevCodeAnalysisIgnore
@@ -8,7 +8,7 @@
 from wimpiggy.test import *
 import wimpiggy.lowlevel as l
 import gtk
-from wimpiggy.error import trap, XError
+from xpra.x11.gtk_x11.error import trap, XError
 
 class TestLowlevel(TestWithSession):
     def root(self, disp=None):
