@@ -1,10 +1,10 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 
 class FakeClient(object):
