@@ -1,23 +1,24 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import gobject
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 
-from xpra.client_base import XpraClientBase, GLibXpraClient
+from xpra.client.gobject_client_base import CommandConnectClient
 
-class TestMemoryClient(GLibXpraClient):
+
+class TestMemoryClient(CommandConnectClient):
     """
         Try to exhaust server memory without authenticating by sending an overly large packet.
     """
 
     def __init__(self, conn, opts):
-        GLibXpraClient.__init__(self, conn, opts)
+        CommandConnectClient.__init__(self, conn, opts)
         def check_connection_dead(*args):
             self.send("irrelevant")
         gobject.timeout_add(1000, check_connection_dead)
@@ -27,13 +28,13 @@ class TestMemoryClient(GLibXpraClient):
         gobject.timeout_add(20*1000, check_connection_timeout)
 
     def make_hello(self, challenge_response=None):
-        capabilities = XpraClientBase.make_hello(self, challenge_response)
-        capabilities["waste_of_space"] = "\0" * (32*1024)
+        capabilities = CommandConnectClient.make_hello(self, challenge_response)
+        capabilities["waste_of_space"] = "\0" * (1024*1024)
         return capabilities
 
     def quit(self, *args):
-        log.info("server correctly terminated the connection")
-        GLibXpraClient.quit(self)
+        log.info("OK: server correctly terminated the connection")
+        CommandConnectClient.quit(self, 0)
 
 def main():
     import sys
