@@ -1,23 +1,24 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010, 2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import gobject
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 
-from xpra.client_base import GLibXpraClient
+from xpra.client.gobject_client_base import CommandConnectClient
 
-class TestIllegalCommandClient(GLibXpraClient):
+
+class TestIllegalCommandClient(CommandConnectClient):
     """
         Sending an illegal command should get us kicked out
     """
 
     def __init__(self, conn, opts):
-        GLibXpraClient.__init__(self, conn, opts)
+        CommandConnectClient.__init__(self, conn, opts)
         def check_kicked_out(*args):
             log.error("BUG: illegal command did not get us kicked out: we are still connected!")
             self.quit()
@@ -29,8 +30,8 @@ class TestIllegalCommandClient(GLibXpraClient):
             self.send("close-window", i)
 
     def quit(self, *args):
-        log.info("server correctly terminated the connection")
-        GLibXpraClient.quit(self)
+        log.info("OK: server correctly terminated the connection")
+        CommandConnectClient.quit(self)
 
 def main():
     import sys
