@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
@@ -9,7 +9,7 @@ import subprocess
 import os.path
 import time
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 HOME = os.path.expanduser("~/")
 
@@ -67,7 +67,7 @@ IPTABLES_CMD = ["sudo", "/usr/sbin/iptables"]
 TRICKLE_BIN = "/usr/bin/trickle"
 TCBENCH = "/opt/VirtualGL/bin/tcbench"
 TCBENCH_LOG = "./tcbench.log"
-XORG_BIN = "/usr/local/bin/Xorg"
+XORG_BIN = "/usr/bin/Xorg"
 VGLRUN_BIN = "/usr/bin/vglrun"
 
 #the glx tests:
@@ -515,17 +515,19 @@ def with_server(start_server_command, stop_server_commands, in_tests, get_stats_
                             cmd = VGLRUN_BIN + test_command
                         elif type(test_command)==list:
                             cmd = [VGLRUN_BIN] + test_command
+                        elif type(test_command)==tuple:
+                            cmd = [VGLRUN_BIN] + list(test_command)
                         else:
                             raise Exception("invalid test command type: %s for %s" % (type(test_command), test_command))
                     else:
                         cmd = test_command
-                        
+
                     print("starting test command: %s, settle time=%s" % (str(cmd), test_command_settle_time))
                     test_command_process = subprocess.Popen(test_command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env, shell=type(cmd)==str)
 
                     if PREVENT_SLEEP:
                         subprocess.Popen(PREVENT_SLEEP_COMMAND)
-                    
+
                     time.sleep(test_command_settle_time)
                     code = test_command_process.poll()
                     assert code is None, "test command %s failed to start: exit code is %s" % (test_command, code)
@@ -649,30 +651,30 @@ def xpra_get_stats(last_record=None):
                 return v
         return default_value
     return [
-            get(["regions_per_second"]),
-            get(["pixels_per_second"]),
-            get(["pixels_encoded_per_second"]),
-            get(["pixels_decoded_per_second"]),
-            get(["batch_delay.min", "min_batch_delay"]),
-            get(["batch_delay.max", "max_batch_delay"]),
-            get(["batch_delay.avg", "avg_batch_delay"]),
-            (int(get(["input_packetcount"], 0))-last_input_packetcount)/MEASURE_TIME,
-            (int(get(["input_bytecount"], 0))-last_input_bytecount)/MEASURE_TIME,
-            (int(get(["output_packetcount"], 0))-last_output_packetcount)/MEASURE_TIME,
-            (int(get(["output_bytecount"], 0))-last_output_bytecount)/MEASURE_TIME,
-            (int(get(["output_mmap_bytecount"], 0))-last_mmap_bytes)/MEASURE_TIME,
-            get(["client_latency.min", "min_client_latency"]),
-            get(["client_latency.max", "max_client_latency"]),
-            get(["client_latency.avg", "avg_client_latency"]),
-            get(["client_ping_latency.min"]),
-            get(["client_ping_latency.max"]),
-            get(["client_ping_latency.avg"]),
-            get(["server_ping_latency.min", "server_latency.min", "min_server_latency"]),
-            get(["server_ping_latency.max", "server_latency.max", "max_server_latency"]),
-            get(["server_ping_latency.avg", "server_latency.avg", "avg_server_latency"]),
-            get(["damage_in_latency.min"]),
-            get(["damage_in_latency.max"]),
-            get(["damage_in_latency.avg"]),
+            get(["encoding.regions_per_second", "regions_per_second"]),
+            get(["encoding.pixels_per_second", "pixels_per_second"]),
+            get(["encoding.pixels_encoded_per_second", "pixels_encoded_per_second"]),
+            get(["encoding.pixels_decoded_per_second", "pixels_decoded_per_second"]),
+            get(["batch.delay.min", "batch_delay.min", "min_batch_delay"]),
+            get(["batch.delay.max", "batch_delay.max", "max_batch_delay"]),
+            get(["batch.delay.avg", "batch_delay.avg", "avg_batch_delay"]),
+            (int(get(["client.connection.input.packetcount", "input_packetcount"], 0))-last_input_packetcount)/MEASURE_TIME,
+            (int(get(["client.connection.input.bytecount", "input_bytecount"], 0))-last_input_bytecount)/MEASURE_TIME,
+            (int(get(["client.connection.output.packetcount", "output_packetcount"], 0))-last_output_packetcount)/MEASURE_TIME,
+            (int(get(["client.connection.output.bytecount", "output_bytecount"], 0))-last_output_bytecount)/MEASURE_TIME,
+            (int(get(["client.connection.output.mmap_bytecount", "output_mmap_bytecount"], 0))-last_mmap_bytes)/MEASURE_TIME,
+            get(["client.latency.min", "client_latency.min", "min_client_latency"]),
+            get(["client.latency.max", "client_latency.max", "max_client_latency"]),
+            get(["client.latency.avg", "client_latency.avg", "avg_client_latency"]),
+            get(["client.ping_latency.min", "client_ping_latency.min"]),
+            get(["client.ping_latency.max", "client_ping_latency.max"]),
+            get(["client.ping_latency.avg", "client_ping_latency.avg"]),
+            get(["server.ping_latency.min", "server_ping_latency.min", "server_latency.min", "min_server_latency"]),
+            get(["server.ping_latency.max", "server_ping_latency.max", "server_latency.max", "max_server_latency"]),
+            get(["server.ping_latency.avg", "server_ping_latency.avg", "server_latency.avg", "avg_server_latency"]),
+            get(["damage.in_latency.min", "damage_in_latency.min"]),
+            get(["damage.in_latency.max", "damage_in_latency.max"]),
+            get(["damage.in_latency.avg", "damage_in_latency.avg"]),
            ]
 
 def get_xpra_start_server_command():
