@@ -1,7 +1,7 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
@@ -20,9 +20,11 @@ class ServerSockInUse(Exception):
 
 class DotXpra(object):
     def __init__(self, sockdir=None, confdir=None):
-        from xpra.platform import get_default_socket_dir, get_default_conf_dir
-        self._confdir = os.path.expanduser(confdir or get_default_conf_dir())
-        self._sockdir = os.path.expanduser(sockdir or get_default_socket_dir())
+        from xpra.platform.paths import get_default_socket_dir, get_default_conf_dir
+        def expand(s):
+            return os.path.expandvars(os.path.expanduser(s))
+        self._confdir = expand(confdir or get_default_conf_dir())
+        self._sockdir = expand(sockdir or get_default_socket_dir())
         if not os.path.exists(self._confdir):
             os.mkdir(self._confdir, o0700)
         if not os.path.exists(self._sockdir):
@@ -42,7 +44,7 @@ class DotXpra(object):
             local_display_name = local_display_name[:local_display_name.rindex(".")]
         assert local_display_name.startswith(":")
         for char in local_display_name[1:]:
-            assert char in "0123456789"
+            assert char in "0123456789", "invalid character in display name: %s" % char
         return local_display_name
 
     def make_path(self, local_display_name, dirpath ):
