@@ -1,6 +1,6 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
@@ -10,74 +10,129 @@ import os
 #before we import xpra.platform
 import platform as python_platform
 assert python_platform
+from xpra.util import AdHocStruct
 
 def warn(msg):
     sys.stderr.write(msg+"\n")
 
-from wimpiggy.util import AdHocStruct
-from wimpiggy.gobject_compat import import_gobject, is_gtk3
-gobject = import_gobject()
-try:
-    from PIL import Image
-    assert Image
-    _has_PIL = True
-except:
-    _has_PIL = False
-ENCODINGS = []
-if is_gtk3():
-    """ with gtk3, we get png via cairo out of the box
-        but we need PIL for the others:
-    """
-    ENCODINGS.append("png")
-    if _has_PIL:
-        ENCODINGS.append("jpeg")
-        ENCODINGS.append("rgb24")
-else:
-    """ with gtk2, we get rgb24 via gdk pixbuf out of the box
-        but we need PIL for the others:
-    """
-    if _has_PIL:
-        ENCODINGS.append("png")
-        ENCODINGS.append("jpeg")
-    ENCODINGS.append("rgb24")
-#we need rgb24 for x264 and vpx (as well as the cython bindings and libraries):
-if "rgb24" in ENCODINGS:
+def codec_import_check(name, top_module, class_module, *classnames):
     try:
-        from xpra import vpx            #@UnusedImport
         try:
-            from xpra.vpx import codec      #@UnusedImport @UnresolvedImport @Reimport
-            ENCODINGS.insert(0, "vpx")
-        except Exception, e:
-            warn("cannot load vpx codec: %s" % e)
-    except ImportError, e:
-        #the vpx module does not exist
-        #xpra was probably built with --without-vpx
-        pass
-    try:
-        from xpra import x264           #@UnusedImport
-        try:
-            from xpra.x264 import codec     #@UnusedImport @UnresolvedImport
-            ENCODINGS.insert(0, "x264")
-        except Exception, e:
-            warn("cannot load x264 codec: %s" % e)
-    except ImportError, e:
-        #the x264 module does not exist
-        #xpra was probably built with --without-x264
-        pass
-    try:
-        bytearray()
-        from xpra.webm.decode import DecodeRGB      #@UnusedImport
-        from xpra.webm.encode import EncodeRGB      #@UnusedImport
-        ENCODINGS.append("webp")
-    except NameError, e:
-        #we need bytearray to use the bindings
-        pass
-    except ImportError, e:
-        #the webm module does not exist
-        #xpra was probably built with --without-webp
-        pass
+            __import__(top_module, {}, {}, [])
+            for classname in classnames:
+                return __import__(class_module, {}, {}, classname)
+        except ImportError, e:
+            #the required module does not exist
+            #xpra was probably built with the option: --without-${name}
+            pass
     except Exception, e:
-        warn("cannot load webp: %s" % e)
+        warn("cannot load %s: %s missing from %s: %s" % (name, classname, class_module, e))
+    return None
+codec_versions = {}
+def add_codec_version(name, top_module, fieldname, invoke=False):
+    try:
+        module = __import__(top_module, {}, {}, [fieldname])
+        if not hasattr(module, fieldname):
+            warn("cannot find %s in %s" % (fieldname, module))
+            return
+        v = getattr(module, fieldname)
+        if invoke and v:
+            v = v()
+        global codec_versions
+        codec_versions[name] = v
+    except ImportError:
+        #not present
+        pass
+
+PIL = codec_import_check("Python Imaging Library", "PIL", "PIL", "Image")
+has_PIL = PIL is not None
+add_codec_version("PIL", "PIL.Image", "VERSION")
+
+enc_vpx = codec_import_check("vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
+has_enc_vpx = enc_vpx is not None
+dec_vpx = codec_import_check("vpx decoder", "xpra.codecs.vpx", "xpra.codecs.vpx.decoder", "Decoder")
+has_dec_vpx = dec_vpx is not None
+add_codec_version("vpx", "xpra.codecs.vpx.encoder", "get_version", True)
+
+enc_x264 = codec_import_check("x264 encoder", "xpra.codecs.enc_x264", "xpra.codecs.enc_x264.encoder", "Encoder")
+has_enc_x264 = enc_x264 is not None
+add_codec_version("x264", "xpra.codecs.enc_x264.encoder", "get_version", True)
+
+enc_nvenc = codec_import_check("nvenc encoder", "xpra.codecs.nvenc", "xpra.codecs.nvenc.encoder", "Encoder")
+has_enc_nvenc = enc_nvenc is not None
+add_codec_version("nvenc", "xpra.codecs.nvenc.encoder", "get_version", True)
+
+csc_swscale = codec_import_check("csc swscale", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
+has_csc_swscale = csc_swscale is not None
+add_codec_version("swscale", "xpra.codecs.csc_swscale.colorspace_converter", "get_version", True)
+
+csc_nvcuda = None   #codec_import_check("csc nvcuda", "xpra.codecs.csc_nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter", "ColorspaceConverter")
+has_csc_nvcuda = csc_nvcuda is not None
+add_codec_version("nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter", "get_version", True)
+
+dec_avcodec = codec_import_check("avcodec decoder", "xpra.codecs.dec_avcodec", "xpra.codecs.dec_avcodec.decoder", "Decoder")
+has_dec_avcodec = dec_avcodec is not None
+add_codec_version("avcodec", "xpra.codecs.dec_avcodec.decoder", "get_version", True)
+
+enc_webp, enc_webp, has_enc_webp_lossless, webp_handlers = None, None, False, None
+try:
+    #python >=2.6 only and required for webp to work:
+    bytearray()
+    try:
+        #these symbols are all available upstream as of libwebp 0.2:
+        enc_webp = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "EncodeRGB", "EncodeRGBA", "EncodeBGR", "EncodeBGRA")
+        dec_webp = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.decode", "DecodeRGB", "DecodeRGBA", "DecodeBGR", "DecodeBGRA")
+        #these symbols were added in libwebp 0.4, and we added HAS_LOSSLESS to the wrapper:
+        _enc_webp_lossless = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "HAS_LOSSLESS", "EncodeLosslessRGB", "EncodeLosslessRGBA", "EncodeLosslessBGRA", "EncodeLosslessBGR")
+        if _enc_webp_lossless:
+            #the fact that the python functions are defined is not enough
+            #we need to check if the underlying C functions actually exist:
+            has_enc_webp_lossless = _enc_webp_lossless.HAS_LOSSLESS
+        add_codec_version("webp", "xpra.codecs.webm", "__VERSION__")
+        webp_handlers = codec_import_check("webp bitmap handler", "xpra.codecs.webm", "xpra.codecs.webm.handlers", "BitmapHandler")
+    except Exception, e:
+        warn("cannot load webp: " % e)
+except:
+    #no bytearray, no webp
+    pass
+has_enc_webp = enc_webp is not None
+has_dec_webp = dec_webp is not None
+
+
+PREFERED_ENCODING_ORDER = ["x264", "vpx", "webp", "png", "png/P", "png/L", "rgb", "jpeg"]
+
+ENCODINGS_TO_NAME = {
+                  "x264"    : "H.264",
+                  "vpx"     : "VPx",
+                  "png"     : "PNG (24/32bpp)",
+                  "png/P"   : "PNG (8bpp colour)",
+                  "png/L"   : "PNG (8bpp grayscale)",
+                  "webp"    : "WebP",
+                  "jpeg"    : "JPEG",
+                  "rgb"     : "Raw RGB + zlib (24/32bpp)",
+                }
+
+ENCODINGS_HELP = {
+                  "x264"    : "H.264 video codec",
+                  "vpx"     : "VPx video codec",
+                  "png"     : "Portable Network Graphics (24 or 32bpp for transparency)",
+                  "png/P"   : "Portable Network Graphics (8bpp colour)",
+                  "png/L"   : "Portable Network Graphics (8bpp grayscale)",
+                  "webp"    : "WebP compression (lossless or lossy)",
+                  "jpeg"    : "JPEG lossy compression",
+                  "rgb"     : "Raw RGB pixels, lossless, compressed using zlib (24 or 32bpp for transparency)",
+                  }
+
+HELP_ORDER = ("x264", "vpx", "webp", "png", "png/P", "png/L", "rgb", "jpeg")
+
+def encodings_help(encodings):
+    h = []
+    for e in HELP_ORDER:
+        if e in encodings:
+            ehelp = ENCODINGS_HELP.get(e)
+            h.append(e.ljust(12) + ehelp)
+    return h
+
 
 ENCRYPTION_CIPHERS = []
 try:
@@ -87,13 +142,18 @@ try:
 except:
     pass
 
+
 def OpenGL_safety_check():
-    if sys.platform.startswith("win"):
-        #at the moment, we can't do double-buffered without some visual
-        #corruption, and win32 only supports double-buffered contexts
-        #so we cannot enable GL on win32!
-        return "not supported on MS Windows"
-    #FIXME: if we fix OpenGL win32 support, the code below may still be useful:
+    #Ubuntu 12.04 will just crash on you if you try:
+    distro = python_platform.linux_distribution()
+    if distro and len(distro)==3 and distro[0]=="Ubuntu":
+        ur = distro[1]  #ie: "12.04"
+        try:
+            rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
+            if rnum<=[12, 04]:
+                return "Ubuntu %s is too buggy" % ur
+        except:
+            pass
     #try to detect VirtualBox:
     #based on the code found here:
     #http://spth.virii.lu/eof2/articles/WarGame/vboxdetect.html
@@ -117,9 +177,9 @@ def OpenGL_safety_check():
         if e.args[0]==errno.EACCES:
             return "VirtualBox is present (VBoxMiniRdrDN)"
     return None
-OPENGL_DEFAULT = False
-#if OpenGL_safety_check() is not None:
-#    OPENGL_DEFAULT = False
+OPENGL_DEFAULT = None       #will auto-detect by probing
+if OpenGL_safety_check() is not None:
+    OPENGL_DEFAULT = False
 
 
 
@@ -178,19 +238,39 @@ def show_codec_help(is_server, speaker_codecs, microphone_codecs):
 def get_build_info():
     info = []
     try:
-        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, REVISION, LOCAL_MODIFICATIONS
+        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
+        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_INFO
         info.append("Built on %s by %s" % (BUILT_ON, BUILT_BY))
         if BUILD_DATE:
             info.append(BUILD_DATE)
-        if int(LOCAL_MODIFICATIONS)==0:
+        try:
+            mods = int(LOCAL_MODIFICATIONS)
+        except:
+            mods = 0
+        if mods==0:
             info.append("revision %s" % REVISION)
         else:
             info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
+        if CYTHON_VERSION!="unknown" or COMPILER_INFO!="unknown":
+            info.append("")
+        if CYTHON_VERSION!="unknown":
+            info.append("built with Cython %s" % CYTHON_VERSION)
+        if COMPILER_INFO!="unknown":
+            info.append(COMPILER_INFO)
     except Exception, e:
-        warn("Error: could not find the build information: %s" % e)
+        warn("Error: could not find the source or build information: %s" % e)
     return info
 
 
+def save_config(conf_file, config, keys):
+    f = open(conf_file, "w")
+    for key in keys:
+        assert key in OPTION_TYPES, "invalid configuration key: %s" % key
+        fn = key.replace("-", "_")
+        v = getattr(config, fn)
+        f.write("%s=%s%s" % (key, v, os.linesep))
+    f.close()
+
 def read_config(conf_file):
     """
         Parses a config file into a dict of strings.
@@ -263,14 +343,14 @@ def read_xpra_defaults():
     """
     #first, read the global defaults:
     if sys.platform.startswith("win"):
-        conf_dir = os.path.dirname(sys.executable)
+        conf_dir = os.path.dirname(os.path.abspath(sys.executable))
     elif sys.prefix == '/usr':
         conf_dir = '/etc/xpra'
     else:
         conf_dir = sys.prefix + '/etc/xpra/'
     defaults = read_xpra_conf(conf_dir)
     #now load the per-user config over it:
-    from xpra.platform import get_default_conf_dir
+    from xpra.platform.paths import get_default_conf_dir
     user_defaults = read_xpra_conf(get_default_conf_dir())
     for k,v in user_defaults.items():
         defaults[k] = v
@@ -285,6 +365,7 @@ OPTION_TYPES = {
                     "username"          : str,
                     "remote-xpra"       : str,
                     "session-name"      : str,
+                    "client-toolkit"    : str,
                     "dock-icon"         : str,
                     "tray-icon"         : str,
                     "window-icon"       : str,
@@ -299,6 +380,7 @@ OPTION_TYPES = {
                     "socket-dir"        : str,
                     "log-file"          : str,
                     "mode"              : str,
+                    "window-layout"     : str,
                     #int options:
                     "quality"           : int,
                     "min-quality"       : int,
@@ -349,19 +431,20 @@ def get_defaults():
     global GLOBAL_DEFAULTS
     if GLOBAL_DEFAULTS is not None:
         return GLOBAL_DEFAULTS
-    from xpra.platform import DEFAULT_SSH_CMD
+    from xpra.platform.features import DEFAULT_SSH_CMD
     try:
         import getpass
         username = getpass.getuser()
     except:
         username = ""
     GLOBAL_DEFAULTS = {
-                    "encoding"          : ENCODINGS[0],
+                    "encoding"          : "",
                     "title"             : "@title@ on @client-machine@",
                     "host"              : "",
                     "username"          : username,
                     "remote-xpra"       : ".xpra/run-xpra",
                     "session-name"      : "",
+                    "client-toolkit"    : "",
                     "dock-icon"         : "",
                     "tray-icon"         : "",
                     "window-icon"       : "",
@@ -378,6 +461,7 @@ def get_defaults():
                     "xvfb"              : "Xvfb +extension Composite -screen 0 3840x2560x24+32 -nolisten tcp -noreset -auth $XAUTHORITY",
                     "socket-dir"        : "",
                     "log-file"          : "$DISPLAY.log",
+                    "window-layout"     : "",
                     "quality"           : -1,
                     "min-quality"       : 50,
                     "speed"             : -1,
@@ -412,7 +496,7 @@ def get_defaults():
                     "opengl"            : OPENGL_DEFAULT,
                     "speaker-codec"     : [],
                     "microphone-codec"  : [],
-                    "key-shortcut"      : ["Meta+Shift+F4:quit"],
+                    "key-shortcut"      : ["Meta+Shift+F4:quit", "Meta+Shift+F8:magic_key"],
                     "bind-tcp"          : None,
                     "start-child"       : None,
                     }
@@ -423,7 +507,6 @@ def validate_in_list(x, options):
         return None
     return "must be in %s" % (", ".join(options))
 OPTIONS_VALIDATION = {
-                    "encoding"          : lambda x : validate_in_list(x, ENCODINGS),
                     "mode"              : lambda x : validate_in_list(x, MODES),
                     }
 #fields that got renamed:
@@ -452,6 +535,13 @@ def parse_bool(k, v):
     else:
         warn("Warning: cannot parse value '%s' for '%s' as a boolean" % (v, k))
 
+def print_bool(k, v):
+    if type(v)==type(None):
+        return 'auto'
+    if type(v)==bool:
+        return v and 'yes' or 'no'
+    warn("Warning: cannot print value '%s' for '%s' as a boolean" % (v, k))
+
 def parse_number(numtype, k, v, auto=-1):
     if type(v)==str:
         v = v.lower()
