@@ -1,8 +1,8 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
@@ -15,19 +15,15 @@ from subprocess import Popen, PIPE
 import signal
 import shlex
 
-import xpra
+from xpra import __version__ as XPRA_VERSION
 from xpra.dotxpra import DotXpra
-from xpra.platform import (XPRA_LOCAL_SERVERS_SUPPORTED,
-                           XPRA_SHADOW_SUPPORTED,
-                           GOT_PASSWORD_PROMPT_SUGGESTION,
-                           add_client_options,
-                           get_default_socket_dir,
-                           init as platform_init)
-from xpra.bytestreams import TwoFileConnection, SocketConnection
-from xpra.scripts.config import ENCODINGS, ENCRYPTION_CIPHERS, make_defaults_struct, show_codec_help, parse_bool
-from wimpiggy.gobject_compat import import_gobject
-
-SIGNAMES = {signal.SIGINT:"SIGINT", signal.SIGTERM:"SIGTERM"}
+from xpra.platform.features import LOCAL_SERVERS_SUPPORTED, SHADOW_SUPPORTED, CAN_DAEMONIZE
+from xpra.platform.options import add_client_options
+from xpra.platform.paths import get_default_socket_dir
+from xpra.platform import init as platform_init
+from xpra.net.bytestreams import TwoFileConnection, SocketConnection
+from xpra.scripts.config import OPTION_TYPES, ENCRYPTION_CIPHERS, \
+    make_defaults_struct, show_codec_help, parse_bool, print_bool, validate_config, encodings_help
 
 
 def warn(msg):
@@ -59,11 +55,11 @@ def main(script_file, cmdline):
     ##
     ## NOTE NOTE NOTE
     #################################################################
-    supports_shadow = XPRA_SHADOW_SUPPORTED
-    supports_server = XPRA_LOCAL_SERVERS_SUPPORTED
+    supports_shadow = SHADOW_SUPPORTED
+    supports_server = LOCAL_SERVERS_SUPPORTED
     if supports_server:
         try:
-            from xpra.wait_for_x_server import wait_for_x_server    #@UnresolvedImport @UnusedImport
+            from xpra.x11.bindings.wait_for_x_server import wait_for_x_server    #@UnresolvedImport @UnusedImport
         except:
             supports_server = False
 
@@ -74,24 +70,29 @@ def main(script_file, cmdline):
                         "\t%prog info [DISPLAY]\n",
                         "\t%prog version [DISPLAY]\n"
                       ]
+    server_modes = []
     if supports_server:
+        server_modes.append("start")
+        server_modes.append("upgrade")
         command_options = ["\t%prog start DISPLAY\n",
                            "\t%prog stop [DISPLAY]\n",
                            "\t%prog list\n",
                            "\t%prog upgrade DISPLAY\n",
                            ] + command_options
     if supports_shadow:
+        server_modes.append("shadow")
         command_options.append("\t%prog shadow DISPLAY\n")
     if not supports_server:
         command_options.append("(This xpra installation does not support starting local servers.)")
 
     hidden_options = {}
-    parser = OptionParser(version="xpra v%s" % xpra.__version__,
+    parser = OptionParser(version="xpra v%s" % XPRA_VERSION,
                           usage="\n" + "".join(command_options))
     defaults = make_defaults_struct()
-    if supports_server or supports_shadow:
+    if len(server_modes):
         group = OptionGroup(parser, "Server Options",
-                    "These options are only relevant on the server when using the 'start', 'upgrade' or 'shadow' mode.")
+                    "These options are only relevant on the server when using the %s mode." %
+                    "or".join(["'%s'" % x for x in server_modes]))
         parser.add_option_group(group)
     if supports_server:
         group.add_option("--start-child", action="append",
@@ -103,7 +104,7 @@ def main(script_file, cmdline):
     else:
         hidden_options["start_child"] = None
         hidden_options["exit_with_children"] = False
-    if supports_server or supports_shadow:
+    if (supports_server or supports_shadow) and CAN_DAEMONIZE:
         group.add_option("--no-daemon", action="store_false",
                           dest="daemon", default=True,
                           help="Don't daemonize when running as a server")
@@ -179,19 +180,20 @@ def main(script_file, cmdline):
     group.add_option("--no-speaker", action="store_false",
                       dest="speaker", default=defaults.speaker,
                       help="Disable forwarding of sound output to the client(s)")
+    CODEC_HELP = """Specify the codec(s) to use for forwarding the %s sound output.
+This parameter can be specified multiple times and the order in which the codecs
+are specified defines the preferred codec order.
+Use the special value 'help' to get a list of options.
+When unspecified, all the available codecs are allowed and the first one is used."""
     group.add_option("--speaker-codec", action="append",
                       dest="speaker_codec", default=defaults.speaker_codec,
-                      help="The audio codec to use for forwarding the speaker sound output "
-                      "(you may specify more than one to define the preferred order, use 'help' to get a list of options, "
-                      "when unspecified all available codecs are allowed and the first one is used)")
+                      help=CODEC_HELP % "speaker")
     group.add_option("--no-microphone", action="store_false",
                       dest="microphone", default=defaults.microphone,
                       help="Disable forwarding of sound input to the server")
     group.add_option("--microphone-codec", action="append",
                       dest="microphone_codec", default=defaults.microphone_codec,
-                      help="The audio codec to use for forwaring the microphone sound input "
-                      "(you may specify more than one to define the preferred order, use 'help' to get a list of options, "
-                      "when unspecified all available codecs are allowed and the first one is used)")
+                      help=CODEC_HELP % "microphone")
 
     group = OptionGroup(parser, "Client Picture Encoding and Compression Options",
                 "These options are used by the client to specify the desired picture and network data compression."
@@ -200,39 +202,25 @@ def main(script_file, cmdline):
     group.add_option("--encoding", action="store",
                       metavar="ENCODING", default=defaults.encoding,
                       dest="encoding", type="str",
-                      help="What image compression algorithm to use: %s." % (", ".join(ENCODINGS)) +
+                      help="What image compression algorithm to use, specify 'help' to get a list of options."
                             " Default: %default."
                       )
-    if "jpeg" in ENCODINGS:
-        group.add_option("-b", "--max-bandwidth", action="store",
-                          dest="max_bandwidth", type="float", default=defaults.max_bandwidth, metavar="BANDWIDTH (kB/s)",
-                          help="Specify the link's maximal receive speed to auto-adjust JPEG quality, 0.0 disables. (default: %default)")
-    else:
-        hidden_options["max_bandwidth"] = 0
-    if len(set(("jpeg", "webp", "x264")).intersection(set(ENCODINGS)))>0:
-        group.add_option("--min-quality", action="store",
-                          metavar="MIN-LEVEL",
-                          dest="min_quality", type="int", default=defaults.min_quality,
-                          help="Sets the minimum x264 encoding quality allowed in automatic quality setting (from 1 to 100, 0 to leave unset). Default: %default.")
-        group.add_option("--quality", action="store",
-                          metavar="LEVEL",
-                          dest="quality", type="int", default=defaults.quality,
-                          help="Use a fixed image compression quality - only relevant to lossy encodings (1-100, 0 to use automatic setting). Default: %default.")
-    else:
-        hidden_options["min_quality"] = defaults.min_quality
-        hidden_options["quality"] = defaults.quality
-    if "x264" in ENCODINGS:
-        group.add_option("--min-speed", action="store",
-                          metavar="SPEED",
-                          dest="min_speed", type="int", default=defaults.min_speed,
-                          help="Sets the minimum x264 encoding speed allowed in automatic speed setting (1-100, 0 to leave unset). Default: %default.")
-        group.add_option("--speed", action="store",
-                          metavar="SPEED",
-                          dest="speed", type="int", default=defaults.speed,
-                          help="Use x264 image compression with the given encoding speed (1-100, 0 to use automatic setting). Default: %default.")
-    else:
-        hidden_options["min_speed"] = defaults.min_speed
-        hidden_options["speed"] = defaults.speed
+    group.add_option("--min-quality", action="store",
+                      metavar="MIN-LEVEL",
+                      dest="min_quality", type="int", default=defaults.min_quality,
+                      help="Sets the minimum encoding quality allowed in automatic quality setting (from 1 to 100, 0 to leave unset). Default: %default.")
+    group.add_option("--quality", action="store",
+                      metavar="LEVEL",
+                      dest="quality", type="int", default=defaults.quality,
+                      help="Use a fixed image compression quality - only relevant to lossy encodings (1-100, 0 to use automatic setting). Default: %default.")
+    group.add_option("--min-speed", action="store",
+                      metavar="SPEED",
+                      dest="min_speed", type="int", default=defaults.min_speed,
+                      help="Sets the minimum encoding speed allowed in automatic speed setting (1-100, 0 to leave unset). Default: %default.")
+    group.add_option("--speed", action="store",
+                      metavar="SPEED",
+                      dest="speed", type="int", default=defaults.speed,
+                      help="Use image compression with the given encoding speed (1-100, 0 to use automatic setting). Default: %default.")
     group.add_option("--auto-refresh-delay", action="store",
                       dest="auto_refresh_delay", type="float", default=defaults.auto_refresh_delay,
                       metavar="DELAY",
@@ -254,13 +242,20 @@ def main(script_file, cmdline):
     parser.add_option_group(group)
     group.add_option("--opengl", action="store",
                       dest="opengl", default=defaults.opengl,
-                      help="Use OpenGL accelerated rendering, options: yes,no,auto. Default: %default.")
+                      help="Use OpenGL accelerated rendering, options: yes,no,auto. Default: %s." % print_bool("opengl", defaults.opengl))
     group.add_option("--no-windows", action="store_false",
                       dest="windows", default=defaults.windows,
                       help="Tells the server not to send any window data, only notifications and bell events will be forwarded (if enabled).")
     group.add_option("--session-name", action="store",
                       dest="session_name", default=defaults.session_name,
                       help="The name of this session, which may be used in notifications, menus, etc. Default: Xpra")
+    group.add_option("--client-toolkit", action="store",
+                      dest="client_toolkit", default=defaults.client_toolkit,
+                      help="The type of client toolkit. Use the value 'help' to get a list of options. Default: %s")
+    group.add_option("--window-layout", action="store",
+                      dest="window_layout", default=defaults.window_layout,
+                      help="The type of window layout to use, each client toolkit may provide different layouts."
+                        "use the value 'help' to get a list of possible layouts. Default: %s")
     group.add_option("--title", action="store",
                       dest="title", default=defaults.title,
                       help="Text which is shown as window title, may use remote metadata variables (default: '%default')")
@@ -270,13 +265,15 @@ def main(script_file, cmdline):
     # let the platform specific code add its own options:
     # adds "--no-tray" for platforms that support it
     add_client_options(group)
+    hidden_options["no_tray"] =  False
+    hidden_options["delay_tray"] =  False
     group.add_option("--tray-icon", action="store",
                           dest="tray_icon", default=defaults.tray_icon,
                           help="Path to the image which will be used as icon for the system-tray or dock")
     group.add_option("--key-shortcut", action="append",
                       dest="key_shortcut", type="str", default=defaults.key_shortcut,
                       help="Define key shortcuts that will trigger specific actions."
-                      + " Defaults to 'Meta+Shift+F4:quit' if no shortcuts are defined.")
+                      + "If no shortcuts are defined, it defaults to '%s'" % (",".join(defaults.key_shortcut or [])))
     group.add_option("--no-keyboard-sync", action="store_false",
                       dest="keyboard_sync", default=defaults.keyboard_sync,
                       help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly")
@@ -332,13 +329,40 @@ def main(script_file, cmdline):
     #ensure all the option fields are set even though
     #some options are not shown to the user:
     for k,v in hidden_options.items():
-        setattr(options, k, v)
+        if not hasattr(options, k):
+            setattr(options, k, v)
+
+    #special handling for URL mode:
+    #xpra attach xpra://[mode:]host:port/?param1=value1&param2=value2
+    if len(args)==2 and args[0]=="attach" and args[1].startswith("xpra://"):
+        url = args[1]
+        from urlparse import urlparse, parse_qs
+        up = urlparse(url)
+        address = up.netloc
+        qpos = url.find("?")
+        if qpos>0:
+            params_str = url[qpos+1:]
+            params = parse_qs(params_str, keep_blank_values=True)
+            f_params = {}
+            #print("params=%s" % str(params))
+            for k,v in params.items():
+                t = OPTION_TYPES.get(k)
+                if t is not None and t!=list:
+                    v = v[0]
+                f_params[k] = v
+            v_params = validate_config(f_params)
+            for k,v in v_params.items():
+                setattr(options, k, v)
+        al = address.lower()
+        if not al.startswith(":") and not al.startswith("tcp") and not al.startswith("ssh"):
+            #assume tcp if not specified
+            address = "tcp:%s" % address
+        args[1] = address
+
     try:
         int(options.dpi)
     except Exception, e:
         parser.error("invalid dpi: %s" % e)
-    if options.encoding and options.encoding not in ENCODINGS:
-        parser.error("encoding %s is not supported, try: %s" % (options.encoding, ", ".join(ENCODINGS)))
     if options.encryption:
         assert len(ENCRYPTION_CIPHERS)>0, "cannot use encryption: no ciphers available"
         if options.encryption not in ENCRYPTION_CIPHERS:
@@ -398,26 +422,30 @@ def main(script_file, cmdline):
         signal.signal(signal.SIGUSR1, sigusr1)
         signal.signal(signal.SIGUSR2, sigusr2)
 
-    if mode=="start" and len(args)>0 and (args[0].startswith("ssh/") or args[0].startswith("ssh:")):
-        #ie: "xpra start ssh:HOST:DISPLAY --start-child=xterm"
-        return run_remote_server(parser, options, args)
-    elif (mode in ("start", "upgrade") and supports_server) or (mode=="shadow" and supports_shadow):
-        nox()
-        from xpra.scripts.server import run_server
-        return run_server(parser, options, mode, script_file, args)
-    elif mode in ("attach", "detach", "screenshot", "version", "info"):
-        return run_client(parser, options, args, mode)
-    elif mode == "stop" and supports_server:
-        nox()
-        return run_stop(parser, options, args)
-    elif mode == "list" and supports_server:
-        return run_list(parser, options, args)
-    elif mode in ("_proxy", "_proxy_start") and supports_server:
-        nox()
-        return run_proxy(parser, options, script_file, args, mode=="_proxy_start")
-    else:
-        parser.error("invalid mode '%s'" % mode)
-        return 1
+    try:
+        if mode=="start" and len(args)>0 and (args[0].startswith("ssh/") or args[0].startswith("ssh:")):
+            #ie: "xpra start ssh:HOST:DISPLAY --start-child=xterm"
+            return run_remote_server(parser, options, args)
+        elif (mode in ("start", "upgrade") and supports_server) or (mode=="shadow" and supports_shadow):
+            nox()
+            from xpra.scripts.server import run_server
+            return run_server(parser, options, mode, script_file, args)
+        elif mode in ("attach", "detach", "screenshot", "version", "info"):
+            return run_client(parser, options, args, mode)
+        elif mode == "stop" and (supports_server or supports_shadow):
+            nox()
+            return run_stop(parser, options, args)
+        elif mode == "list" and (supports_server or supports_shadow):
+            return run_list(parser, options, args)
+        elif mode in ("_proxy", "_proxy_start") and (supports_server or supports_shadow):
+            nox()
+            return run_proxy(parser, options, script_file, args, mode=="_proxy_start")
+        else:
+            parser.error("invalid mode '%s'" % mode)
+            return 1
+    except KeyboardInterrupt, e:
+        sys.stderr.write("\ncaught %s, exiting\n" % repr(e))
+        return 128+signal.SIGINT
 
 
 def parse_display_name(error_cb, opts, display_name):
@@ -510,10 +538,10 @@ def pick_display(parser, opts, extra_args):
     else:
         parser.error("too many arguments")
 
-def _socket_connect(sock, endpoint, description):
+def _socket_connect(sock, endpoint, description, dtype):
     sock.connect(endpoint)
     sock.settimeout(None)
-    return SocketConnection(sock, sock.getsockname(), sock.getpeername(), description)
+    return SocketConnection(sock, sock.getsockname(), sock.getpeername(), description, dtype)
 
 def connect_or_fail(display_desc):
     try:
@@ -524,7 +552,7 @@ def connect_or_fail(display_desc):
 def ssh_connect_failed(message):
     #by the time ssh fails, we may have entered the gtk main loop
     #(and more than once thanks to the clipboard code..)
-    from wimpiggy.util import gtk_main_quit_really
+    from xpra.gtk_common.quit import gtk_main_quit_really
     gtk_main_quit_really()
 
 def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
@@ -545,6 +573,7 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
         cmd += display_desc["remote_xpra"] + display_desc["proxy_command"] + display_desc["display_as_args"]
         try:
             kwargs = {}
+            kwargs["stderr"] = sys.stderr
             if os.name=="posix" and not sys.platform.startswith("darwin"):
                 def setsid():
                     #run in a new session
@@ -554,10 +583,11 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
                 from subprocess import CREATE_NEW_PROCESS_GROUP, CREATE_NEW_CONSOLE
                 flags = CREATE_NEW_PROCESS_GROUP | CREATE_NEW_CONSOLE
                 kwargs["creationflags"] = flags
+                kwargs["stderr"] = PIPE
             if debug_cb:
                 debug_cb("starting %s tunnel" % str(cmd[0]))
                 #debug_cb("starting ssh: %s with kwargs=%s" % (str(cmd), kwargs))
-            child = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE, **kwargs)
+            child = Popen(cmd, stdin=PIPE, stdout=PIPE, **kwargs)
         except OSError, e:
             raise Exception("Error running ssh program '%s': %s" % (cmd, e))
         def abort_test(action):
@@ -595,37 +625,24 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
                         raise Exception("cannot find function to kill subprocess")
             except Exception, e:
                 print("error trying to stop ssh tunnel process: %s" % e)
-        return TwoFileConnection(child.stdin, child.stdout, abort_test, target=display_name, info="SSH", close_cb=stop_tunnel)
+        return TwoFileConnection(child.stdin, child.stdout, abort_test, target=display_name, info=dtype, close_cb=stop_tunnel)
 
     elif dtype == "unix-domain":
         sockdir = DotXpra(display_desc["socket_dir"])
         sock = socket.socket(socket.AF_UNIX)
         sock.settimeout(5)
         sockfile = sockdir.socket_path(display_desc["display"])
-        return _socket_connect(sock, sockfile, display_name)
+        return _socket_connect(sock, sockfile, display_name, dtype)
 
     elif dtype == "tcp":
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.settimeout(10)
         tcp_endpoint = (display_desc["host"], display_desc["port"])
-        return _socket_connect(sock, tcp_endpoint, display_name)
+        return _socket_connect(sock, tcp_endpoint, display_name, dtype)
 
     else:
         assert False, "unsupported display type in connect: %s" % dtype
 
-def set_signal_handlers(app):
-    gobject = import_gobject()
-    def deadly_signal(signum, frame):
-        print("got deadly signal %s, exiting" % SIGNAMES.get(signum, signum))
-        app.cleanup()
-        os._exit(128 + signum)
-    def app_signal(signum, frame):
-        print("\ngot signal %s, exiting" % SIGNAMES.get(signum, signum))
-        signal.signal(signal.SIGINT, deadly_signal)
-        signal.signal(signal.SIGTERM, deadly_signal)
-        gobject.timeout_add(0, app.quit, 128 + signum, priority=gobject.PRIORITY_HIGH)
-    signal.signal(signal.SIGINT, app_signal)
-    signal.signal(signal.SIGTERM, app_signal)
 
 def run_client(parser, opts, extra_args, mode):
     if mode=="screenshot":
@@ -634,51 +651,111 @@ def run_client(parser, opts, extra_args, mode):
         screenshot_filename = extra_args[0]
         extra_args = extra_args[1:]
 
-    if mode in ("attach"):
-        sys.stdout.write("xpra client version %s\n" % xpra.__version__)
-        sys.stdout.flush()
-
-    conn = connect_or_fail(pick_display(parser, opts, extra_args))
     if opts.compression_level < 0 or opts.compression_level > 9:
         parser.error("Compression level must be between 0 and 9 inclusive.")
     if opts.quality!=-1 and (opts.quality < 0 or opts.quality > 100):
         parser.error("Quality must be between 0 and 100 inclusive. (or -1 to disable)")
 
-    if mode=="screenshot":
-        from xpra.client_base import ScreenshotXpraClient
-        app = ScreenshotXpraClient(conn, opts, screenshot_filename)
-    elif mode=="info":
-        from xpra.client_base import InfoXpraClient
-        app = InfoXpraClient(conn, opts)
-    elif mode=="version":
-        from xpra.client_base import VersionXpraClient
-        app = VersionXpraClient(conn, opts)
-    else:
-        from xpra.client import XpraClient
-        app = XpraClient(conn, opts)
-    return do_run_client(app, conn.target, mode)
+    def connect():
+        return connect_or_fail(pick_display(parser, opts, extra_args))
 
-def do_run_client(app, target, mode):
-    def got_gibberish_msg(obj, data):
-        if str(data).find("assword")>0:
-            sys.stdout.write("Your ssh program appears to be asking for a password.\n"
-                             + GOT_PASSWORD_PROMPT_SUGGESTION)
+    if mode=="screenshot":
+        from xpra.client.gobject_client_base import ScreenshotXpraClient
+        app = ScreenshotXpraClient(connect(), opts, screenshot_filename)
+    elif mode=="info":
+        from xpra.client.gobject_client_base import InfoXpraClient
+        app = InfoXpraClient(connect(), opts)
+    elif mode=="version":
+        from xpra.client.gobject_client_base import VersionXpraClient
+        app = VersionXpraClient(connect(), opts)
+    else:
+        if mode in ("attach"):
+            sys.stdout.write("xpra client version %s\n" % XPRA_VERSION)
             sys.stdout.flush()
-        if str(data).find("login")>=0:
-            sys.stdout.write("Your ssh program appears to be asking for a username.\n"
-                             "Perhaps try using something like 'ssh:USER@host:display'?\n")
-            sys.stdout.flush()
-    app.connect("received-gibberish", got_gibberish_msg)
-    def handshake_complete(*args):
-        from wimpiggy.log import Logger
-        log = Logger()
-        if mode=="detach":
-            log.info("handshake-complete: detaching")
-            app.quit(0)
-        elif mode=="attach":
-            log.info("Attached to %s (press Control-C to detach)\n" % target)
-    app.connect("handshake-complete", handshake_complete)
-    set_signal_handlers(app)
+        app = make_client(parser.error, opts)
+        if opts.encoding:
+            if opts.encoding=="rgb24":
+                #fix old encoding name
+                opts.encoding = "rgb"
+            err = opts.encoding not in app.get_encodings()
+            if err:
+                print("invalid encoding: %s" % opts.encoding)
+            if opts.encoding=="help" or err:
+                print("%s xpra client supports the following encodings:\n * %s" % (app.client_toolkit(), "\n * ".join(encodings_help(app.get_encodings()))))
+                if err:
+                    return 1
+                return 0
+        layouts = app.get_supported_window_layouts() or ["default"]
+        layouts_str = ", ".join(layouts)
+        if opts.window_layout and opts.window_layout.lower()=="help":
+            print("%s supports the following layouts: %s" % (app.client_toolkit(), layouts_str))
+            return 0
+        if opts.window_layout not in layouts:
+            parser.error("window layout '%s' is not supported by the %s toolkit, valid options are: %s" % (opts.window_layout, app.client_toolkit(), layouts_str))
+        conn = connect()
+        def handshake_complete(*args):
+            from xpra.log import Logger
+            log = Logger()
+            if mode=="detach":
+                log.info("handshake-complete: detaching")
+                app.quit(0)
+            elif mode=="attach":
+                log.info("Attached to %s (press Control-C to detach)\n" % conn.target)
+        if hasattr(app, "connect"):
+            app.connect("handshake-complete", handshake_complete)
+        app.setup_connection(conn)
+        app.init(opts)
+    return do_run_client(app)
+
+def make_client(error_cb, opts):
+    app = None
+    if not opts.client_toolkit:
+        from xpra.gtk_common.gobject_compat import import_gobject, is_gtk3
+        import_gobject()
+        if is_gtk3():
+            opts.client_toolkit = "gtk3"
+        else:
+            opts.client_toolkit = "gtk2"
+
+    ct = opts.client_toolkit.lower()
+    toolkits = {}
+    try:
+        import gtk.gdk                      #@UnusedImport
+        import xpra.client.gtk2             #@UnusedImport
+        toolkits["gtk2"] = "xpra.client.gtk2.client"
+    except:
+        pass
+    try:
+        from gi.repository import Gtk       #@UnresolvedImport @UnusedImport
+        import xpra.client.gtk3             #@UnusedImport
+        toolkits["gtk3"] = "xpra.client.gtk3.client"
+    except:
+        pass
+    try:
+        from PyQt4 import QtCore, QtGui     #@UnresolvedImport @UnusedImport
+        try:
+            import xpra.client.qt4              #@UnusedImport
+            toolkits["qt4"] = "xpra.client.qt4.client"
+        except Exception, e:
+            print("failed to load qt client: %s", e)
+            pass
+    except ImportError, e:
+        #qt not installed
+        pass
+    if ct=="help":
+        error_cb("The following client toolkits are available: %s" % (", ".join(toolkits.keys())))
+    client_module = toolkits.get(ct)
+    if client_module is None:
+        error_cb("invalid client toolkit: %s, try one of: %s" % (
+                    opts.client_toolkit, ", ".join(toolkits.keys())))
+    toolkit_module = __import__(client_module, globals(), locals(), ['XpraClient'])
+    if toolkit_module is None:
+        error_cb("could not load %s" % client_module)
+    if not opts.window_layout:
+        opts.window_layout = "default"
+    return toolkit_module.XpraClient()
+
+def do_run_client(app):
     try:
         try:
             return app.run()
@@ -701,15 +778,16 @@ def run_remote_server(parser, opts, args):
     #and use _proxy_start subcommand:
     params["proxy_command"] = ["_proxy_start"]
     conn = connect_or_fail(params)
-    from xpra.client import XpraClient
-    app = XpraClient(conn, opts)
-    do_run_client(app, params["display_name"], "attach")
+    app = make_client(parser.error, opts)
+    app.setup_connection(conn)
+    app.init(opts)
+    do_run_client(app)
 
 def run_proxy(parser, opts, script_file, args, start_server=False):
-    from xpra.proxy import XpraProxy
+    from xpra.server.proxy import XpraProxy
     assert "gtk" not in sys.modules
     if start_server:
-        assert len(args)==1
+        assert len(args)==1, "proxy-start: expected 1 argument but got %s" % len(args)
         display_name = args[0]
         #we must use a subprocess to avoid messing things up - yuk
         cmd = [script_file, "start"]+args
@@ -737,7 +815,7 @@ def run_proxy(parser, opts, script_file, args, start_server=False):
 
 def run_stop(parser, opts, extra_args):
     assert "gtk" not in sys.modules
-    from xpra.client_base import StopXpraClient
+    from xpra.client.gobject_client_base import StopXpraClient
 
     def show_final_state(display):
         sockdir = DotXpra(opts.socket_dir)
