@@ -1,7 +1,7 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # DO NOT IMPORT GTK HERE: see
@@ -55,7 +55,7 @@ class ChildReaper(object):
         self._quit = quit_cb
         self._children_pids = children_pids
         self._dead_pids = set()
-        from wimpiggy.log import Logger
+        from xpra.log import Logger
         self._logger = Logger()
 
     def check(self):
@@ -87,14 +87,14 @@ class ChildReaper(object):
 
 def save_pid(pid):
     import gtk
-    import wimpiggy.prop
-    wimpiggy.prop.prop_set(gtk.gdk.get_default_root_window(),
+    from xpra.x11.gtk_x11.prop import prop_set
+    prop_set(gtk.gdk.get_default_root_window(),
                            "_XPRA_SERVER_PID", "u32", pid)
 
 def get_pid():
     import gtk
-    import wimpiggy.prop
-    return wimpiggy.prop.prop_get(gtk.gdk.get_default_root_window(),
+    from xpra.x11.gtk_x11.prop import prop_get
+    return prop_get(gtk.gdk.get_default_root_window(),
                                   "_XPRA_SERVER_PID", "u32")
 
 def sh_quotemeta(s):
@@ -209,6 +209,11 @@ def close_all_fds(exceptions=[]):
 def run_server(parser, opts, mode, xpra_file, extra_args):
     if len(extra_args) != 1:
         parser.error("need exactly 1 extra argument")
+    if opts.encoding and opts.encoding=="help":
+        from xpra.scripts.config import encodings_help
+        from xpra.server.server_base import SERVER_ENCODINGS
+        print("server supports the following encodings:\n * %s" % ("\n * ".join(encodings_help(SERVER_ENCODINGS))))
+        return 0
     assert mode in ("start", "upgrade", "shadow")
     upgrading = mode == "upgrade"
     shadowing = mode == "shadow"
@@ -302,7 +307,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     scriptfile.write(xpra_runner_shell_script(xpra_file, starting_dir, opts.socket_dir))
     scriptfile.close()
 
-    from wimpiggy.log import Logger
+    from xpra.log import Logger
     log = Logger()
 
     # Initialize the sockets before the display,
@@ -313,7 +318,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         def setup_tcp_socket(bind_to):
             try:
                 tcp_socket = create_tcp_socket(parser, bind_to)
-                sockets.append(tcp_socket)
+                sockets.append(("tcp", tcp_socket))
                 def cleanup_tcp_socket():
                     log.info("closing tcp socket %s", bind_to)
                     try:
@@ -336,7 +341,10 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
             parser.error("You already have an xpra server running at %s\n"
                          "  (did you want 'xpra upgrade'?)"
                          % (display_name,))
-        sockets.append(create_unix_domain_socket(sockpath, opts.mmap_group))
+        except Exception, e:
+            parser.error("socket path error: %s" % e)
+        sock = create_unix_domain_socket(sockpath, opts.mmap_group)
+        sockets.append(("unix-domain", sock))
         def cleanup_socket():
             log.info("removing socket %s", sockpath)
             try:
@@ -396,13 +404,13 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
             if os.name=="posix":
                 os.setsid()
         try:
-            xvfb = subprocess.Popen(xvfb_cmd+[display_name], executable=xvfb_executable, close_fds=True, preexec_fn=setsid)
+            xvfb = subprocess.Popen(xvfb_cmd+[display_name], executable=xvfb_executable, close_fds=True,
+                                    stdin=subprocess.PIPE, preexec_fn=setsid)
         except OSError, e:
             sys.stderr.write("Error starting Xvfb: %s\n" % (e,))
             return  1
-        raw_cookie = os.urandom(16)
-        baked_cookie = raw_cookie.encode("hex")
-        xauth_cmd = ["xauth", "add", display_name, "MIT-MAGIC-COOKIE-1", baked_cookie]
+        from xpra.os_util import get_hex_uuid
+        xauth_cmd = ["xauth", "add", display_name, "MIT-MAGIC-COOKIE-1", get_hex_uuid()]
         try:
             code = subprocess.call(xauth_cmd)
             if code != 0:
@@ -429,8 +437,8 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if xvfb_error():
         return  1
 
-    if not sys.platform.startswith("win"):
-        from xpra.wait_for_x_server import wait_for_x_server        #@UnresolvedImport
+    if not sys.platform.startswith("win") and not sys.platform.startswith("darwin"):
+        from xpra.x11.bindings.wait_for_x_server import wait_for_x_server        #@UnresolvedImport
         # Whether we spawned our server or not, it is now running -- or at least
         # starting.  First wait for it to start up:
         try:
@@ -455,41 +463,44 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         import gtk          #@Reimport
 
     if shadowing:
-        xvfb_pid = None
-    elif clobber:
-        xvfb_pid = get_pid()
+        xvfb_pid = None     #we don't own the display
+        from xpra.platform.shadow_server import ShadowServer
+        app = ShadowServer()
+        app.init(sockets, opts)
     else:
-        if xvfb_error(True):
-            return  1
-        xvfb_pid = xvfb.pid
-        if xvfb_pid is not None:
-            save_pid(xvfb_pid)
+        from xpra.x11.gtk_x11 import gdk_display_source
+        assert gdk_display_source
 
-    if shadowing:
-        if sys.platform.startswith("win"):
-            from xpra.win32.shadow_server import XpraWin32ShadowServer
-            app = XpraWin32ShadowServer(sockets, opts)
+        if clobber:
+            #get the saved pid:
+            xvfb_pid = get_pid()
         else:
-            from xpra.shadow_server import XpraX11ShadowServer
-            app = XpraX11ShadowServer(sockets, opts)
-    else:
+            #check that the vfb has started ok:
+            if xvfb_error(True):
+                return  1
+            xvfb_pid = xvfb.pid
+
         #check for an existing window manager:
-        from wimpiggy.wm import wm_check
-        if not wm_check(display):
+        from xpra.x11.gtk_x11.wm import wm_check
+        if not wm_check(display, upgrading):
             return 1
         try:
-            from wimpiggy.lowlevel import displayHasXComposite     #@UnresolvedImport
             # This import is delayed because the module depends on gtk:
-            from xpra.server import XpraServer
+            from xpra.x11.server import XpraServer
+            from xpra.x11.bindings.window_bindings import X11WindowBindings     #@UnresolvedImport
+            X11Window = X11WindowBindings()
         except ImportError, e:
             log.error("Failed to load Xpra server components, check your installation: %s" % e)
             return 1
-        root = gtk.gdk.get_default_root_window()
-        if not displayHasXComposite(root):
+        if not X11Window.displayHasXComposite():
             log.error("Xpra is a compositing manager, it cannot use a display which lacks the XComposite extension!")
             return 1
-        app = XpraServer(clobber, sockets, opts)
+        app = XpraServer()
+        app.init(clobber, sockets, opts)
 
+
+    if xvfb_pid is not None:
+        save_pid(xvfb_pid)
     #we got this far so the sockets have initialized and
     #the server should be able to manage the display
     #from now on, if we exit without upgrading we will also kill the Xvfb
@@ -540,7 +551,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         log("upgrading=%s, shadowing=%s, pulseaudio=%s, pulseaudio_command=%s",
                  upgrading, shadowing, opts.pulseaudio, opts.pulseaudio_command)
         if not upgrading and not shadowing and opts.pulseaudio and len(opts.pulseaudio_command)>0:
-            pa_proc = subprocess.Popen(opts.pulseaudio_command, shell=True, close_fds=True)
+            pa_proc = subprocess.Popen(opts.pulseaudio_command, stdin=subprocess.PIPE, shell=True, close_fds=True)
             procs.append(pa_proc)
             log.info("pulseaudio server started with pid %s", pa_proc.pid)
             def check_pa_start():
@@ -569,7 +580,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         for child_cmd in opts.start_child:
             if child_cmd:
                 try:
-                    proc = subprocess.Popen(child_cmd, env=env, shell=True, close_fds=True)
+                    proc = subprocess.Popen(child_cmd, stdin=subprocess.PIPE, env=env, shell=True, close_fds=True)
                     children_pids[proc.pid] = child_cmd
                     procs.append(proc)
                 except OSError, e:
@@ -583,6 +594,9 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         e = app.run()
     except KeyboardInterrupt:
         e = 0
+    except Exception, e:
+        log.error("server error", exc_info=True)
+        e = 128
     if e>0:
         log.info("upgrading: not cleaning up Xvfb or socket")
         # Upgrading, so leave X server running
