@@ -16,11 +16,13 @@ from xpra.server.shadow_server_base import RootWindowModel
 
 class GTKRootWindowModel(RootWindowModel):
 
-    def get_image(self, x, y, width, height):
-        v = get_rgb_rawdata(self.window, x, y, width, height, logger=log)
+    def get_image(self, x, y, width, height, logger=None):
+        v = get_rgb_rawdata(self.window, x, y, width, height, logger=logger)
         if v is None:
             return None
-        return ImageWrapper(*v)
+        args = list(v)
+        args.append(0)
+        return ImageWrapper(*args)
 
     def take_screenshot(self):
         log("grabbing screenshot")
