@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
@@ -9,7 +9,7 @@ import os.path
 
 from xpra.scripts.exec_util import safe_exec
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 
 
@@ -44,12 +44,12 @@ def get_pactl_bin():
 			pactl_bin = which("pactl")
 	return pactl_bin
 
-def pactl_output(pactl_cmd):
+def pactl_output(*pactl_args):
 	pactl_bin = get_pactl_bin()
 	if not pactl_bin:
 		return -1, None
 	#ie: "pactl list"
-	cmd = [pactl_bin, pactl_cmd]
+	cmd = [pactl_bin] + list(pactl_args)
 	try:
 		code, out, _ = safe_exec(cmd)
 		return  code, out
@@ -60,9 +60,9 @@ def pactl_output(pactl_cmd):
 def get_x11_property(atom_name):
 	if sys.platform.startswith("win"):
 		return ""
-	from gtk import gdk
-	root = gdk.get_default_root_window()
 	try:
+		from gtk import gdk
+		root = gdk.get_default_root_window()
 		pulse_server_atom = gdk.atom_intern(atom_name)
 		p = root.property_get(pulse_server_atom)
 		if p is None:
@@ -91,15 +91,29 @@ def has_pa():
 	return has_pulseaudio
 
 
-def get_pactl_server():
+def set_source_mute(device, mute=False):
+	code, out = pactl_output("set-source-mute", device, str(int(mute)))
+	log("set_source_mute: output=%s", out)
+	return code==0
+
+def get_pactl_stat_line(prefix):
+	if not has_pa():
+		return ""
 	code, out = pactl_output("stat")
 	if code!=0:
 		return	""
 	for line in out.splitlines():
-		if line.startswith("Server String: "):
-			return line[len("Server String: "):]
+		if line.startswith(prefix):
+			return line[len(prefix):].strip()
 	return ""
 
+def get_default_sink():
+	return get_pactl_stat_line("Default Sink:")
+
+def get_pactl_server():
+	return get_pactl_stat_line("Server String:")
+
+
 def get_pulse_server(may_start_it=True):
 	xp = get_x11_property("PULSE_SERVER")
 	if xp or not may_start_it:
