@@ -1,35 +1,28 @@
-#!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
-import gobject
+import time
 
+from xpra.signal_object import SignalObject
 from xpra.sound.gstreamer_util import gst
-from wimpiggy.util import AutoPropGObjectMixin, one_arg_signal
-from wimpiggy.log import Logger
+from xpra.log import Logger, debug_if_env
 log = Logger()
-
-DEBUG_SOUND = os.environ.get("XPRA_SOUND_DEBUG", "0")=="1"
-if DEBUG_SOUND:
-    debug = log.info
-else:
-    debug = log.debug
+debug = debug_if_env(log, "XPRA_SOUND_DEBUG")
 
 
-class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
+class SoundPipeline(SignalObject):
 
-    __gsignals__ = {
-        "state-changed": one_arg_signal,
-        "bitrate-changed": one_arg_signal,
-        "error": one_arg_signal,
-        }
+    __generic_signals__ = [
+        "state-changed",
+        "bitrate-changed",
+        "error"
+        ]
 
     def __init__(self, codec):
-        super(gobject.GObject, self).__init__()
-        super(AutoPropGObjectMixin, self).__init__()
+        SignalObject.__init__(self)
+        self.add_signals(SoundPipeline.__generic_signals__)
         self.codec = codec
         self.codec_description = codec
         self.codec_mode = ""
@@ -37,12 +30,29 @@ class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
         self.bus_message_handler_id = None
         self.bitrate = -1
         self.pipeline = None
+        self.start_time = 0
         self.state = "stopped"
+        self.buffer_count = 0
+        self.byte_count = 0
+
+    def get_info(self):
+        info = {"codec"             : self.codec,
+                "codec_description" : self.codec_description,
+                "state"             : self.get_state(),
+                "buffers"           : self.buffer_count,
+                "bytes"             : self.byte_count,
+                }
+        if self.codec_mode:
+            info["codec_mode"] = self.codec_mode
+        if self.bitrate>0:
+            info["speaker.bitrate"] = self.bitrate
+        return info
 
     def setup_pipeline_and_bus(self, elements):
         debug("pipeline elements=%s", elements)
         pipeline_str = " ! ".join([x for x in elements if x is not None])
         debug("pipeline=%s", pipeline_str)
+        self.start_time = time.time()
         self.pipeline = gst.parse_launch(pipeline_str)
         self.bus = self.pipeline.get_bus()
         self.bus_message_handler_id = self.bus.connect("message", self.on_message)
@@ -51,11 +61,10 @@ class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
     def do_get_state(self, state):
         if not self.pipeline:
             return  "stopped"
-        if state==gst.STATE_PLAYING:
-            return  "active"
-        if state==gst.STATE_NULL:
-            return  "stopped"
-        return  "unknown"
+        return {gst.STATE_PLAYING   : "active",
+                gst.STATE_PAUSED    : "paused",
+                gst.STATE_NULL      : "stopped",
+                gst.STATE_READY     : "ready"}.get(state, "unknown")
 
     def get_state(self):
         return self.state
@@ -67,9 +76,6 @@ class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
         log("new bitrate: %s", self.bitrate)
         #self.emit("bitrate-changed", new_bitrate)
 
-    def get_bitrate(self):
-        return self.bitrate
-
     def start(self):
         debug("SoundPipeline.start()")
         self.state = "active"
@@ -84,6 +90,7 @@ class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
 
     def cleanup(self):
         debug("SoundPipeline.cleanup()")
+        SignalObject.cleanup(self)
         self.stop()
         if self.bus:
             self.bus.remove_signal_watch()
@@ -147,13 +154,10 @@ class SoundPipeline(AutoPropGObjectMixin, gobject.GObject):
             d = message.parse_duration()
             debug("duration changed: %s", d)
         elif t == gst.MESSAGE_LATENCY:
-            log.info("Latency message from %s: %s", message.src, message)            
+            log.info("Latency message from %s: %s", message.src, message)
         elif t == gst.MESSAGE_WARNING:
             w = message.parse_warning()
             log.warn("pipeline warning: %s", w[0].message)
             log.info("pipeline warning: %s", w[1:])
         else:
             log.info("unhandled bus message type %s: %s / %s", t, message, message.structure)
-
-
-gobject.type_register(SoundPipeline)
